#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ^^^ just in case this is executed by Python 2

#   Copyright (c) 2012-2014 Tobias Leupold <tobias.leupold@web.de>
#
#   This file is part of the muckturnier package.
#
#   This program is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation in version 2 of the License.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
#   for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

"""This checks if we have everything we need to run muckturnier and sets up the main window."""

import sys

# Is this executed by Python 3?
if sys.version_info[0] < 3:
    from Tkinter import Tk
    import tkMessageBox
    mainWindow = Tk()
    mainWindow.withdraw()
    tkMessageBox.showerror(
        'muckturnier',
        (
            # We use the old %-format here in case this is executed by Python <= 2.5
            'muckturnier kann nicht gestartet werden: Es wird Python 3 benötigt. Die gestartete '
            'Python-Version ist %s.%s.\n\n'
            'Bitte Python 3 installieren bzw. die bestehende Installation überprüfen!\n\n'
            'Python 3 kann von http://python.org/ heruntergeladen werden.'
        ) % (sys.version_info[0], sys.version_info[1])
    )
    sys.exit(1)

# We're on Python 3. Let's see if everything else is also okay.

# Let's define a (graphical) tkinter error message we display in case something is wrong
# (hoping we do have the tkinter modules. If not, it's the user's problem ;-)
def abort(error):
    from tkinter import Tk, messagebox
    mainWindow = Tk()
    mainWindow.withdraw()
    messagebox.showerror('muckturnier', error)
    sys.exit(1)

# Check if Python has SQLite 3 support
try:
    from sqlite3 import version
except:
    abort(
        'muckturnier kann nicht gestartet werden: Das Python-Modul für den Zugriff auf '
        'SQLite-3-Datenbanken wurde nicht gefunden.\n\n'
        'Bitte das Modul „sqlite3“ installieren, die bestehende Installation überprüfen oder '
        'Python 3 neu installieren.\n\n'
        'Python 3 kann von http://python.org/ heruntergeladen werden.'
    )

# The Python stuff seems to be working so far.

# Check if we have PyQt5.
try:
    from PyQt5.QtCore import QT_VERSION_STR
except:
    abort(
        'muckturnier kann nicht gestartet werden: Das Paket PyQt5 wurde nicht gefunden.\n\n'
        'Bitte PyQt5 installieren bzw. die bestehende Installation überprüfen!\n\n'
        'PyQt5 kann von http://riverbankcomputing.co.uk/software/pyqt/ heruntergeladen werden.'
    )

# We found at least the QtCore module. We also need QtWidgets, so let's see ...
try:
    from PyQt5.QtWidgets import QApplication
except:
    abort(
        'muckturnier kann nicht gestartet werden: Das Qt-Modul „QtWidgets“ wurde nicht '
        'gefunden.\n\n'
        'Bitte das Modul QtWidgets installieren, die PyQt5-Installation überprüfen bzw. PyQt5 neu '
        'installieren.\n\n'
        'PyQt5 kann von http://riverbankcomputing.co.uk/software/pyqt/ heruntergeladen werden.'
    )

# We need QtGui as well ...
try:
    from PyQt5.QtGui import QColor
except:
    abort(
        'muckturnier kann nicht gestartet werden: Das Qt-Modul „QtGui“ wurde nicht gefunden.\n\n'
        'Bitte das Modul QtGui installieren, die PyQt5-Installation überprüfen bzw. PyQt5 neu '
        'installieren.\n\n'
        'PyQt5 kann von http://riverbankcomputing.co.uk/software/pyqt/ heruntergeladen werden.'
    )

# Qt 5 seems to be okay.

# All we still need is the muckturnier package itself, so let's see ...
try:
    from muckturnier.MainWindow import MainWindow
except:
    abort((
        'muckturnier kann nicht gestartet werden: Es fehlen Programmdateien oder es ist ein '
        'unerwarteter Fehler aufgetreten. Die Fehlermeldung war:\n\n{}\n\n'
        'Bitte die Installation überprüfen oder muckturnier neu installieren! muckturnier kann '
        'von http://nasauber.de/opensource/muckturnier/ heruntergeladen werden.\n\n'
        'Wenn dieser Fehler bei einer neuen Installation auftritt, dann bitte den Fehler melden, '
        'am besten per E-Mail an <tobias.leupold@web.de>.'
    ).format(sys.exc_info()[1]))

# Everything is fine, so let's go :-)

app = QApplication(sys.argv)

mainWindow = MainWindow()
mainWindow.show()

if len(sys.argv) > 1:
    if '-h' in sys.argv or '-?' in sys.argv or '--help' in sys.argv:
        print('Usage: {} [Tournament database to load]'.format(sys.argv[0]))
        sys.exit(0)
    mainWindow.loadTournament(False, sys.argv[1])

# Start the main event loop
sys.exit(app.exec_())
