#   Copyright (c) 2012-2014 Tobias Leupold <tobias.leupold@web.de>
#
#   This file is part of the muckturnier package.
#
#   This program is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation in version 2 of the License.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
#   for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import os
import sqlite3

from muckturnier.enums import MT, DB
from muckturnier.shared import shared

class Database:

    """This is the storage backend managing all communication with the database. Additionally
    provides functions that depend on settings made or the current state of the dataset."""

    dbPointer = False

    def instance():
        """Returns a pointer to the database object. Creates one if we don't have one yet."""
        if Database.dbPointer == False:
            Database.dbPointer = Database()
        return Database.dbPointer

    def __init__(self):
        """Constructs the database object."""
        self.settings = {}

    def create(self, dbFile):
        """Creates a new database in dbFile."""

        if os.path.exists(dbFile):
            os.remove(dbFile)

        self.db = sqlite3.connect(dbFile)
        self.query = self.db.cursor()

        self.query.execute((
            'CREATE TABLE config(\n'
            '    key TEXT,\n'
            '    value TEXT\n'
            ')'
        ))
        self.query.execute((
            'CREATE TABLE players(\n'
            '    id INTEGER PRIMARY KEY,\n'
            '    name TEXT\n'
            ')'
        ))
        self.query.execute((
            'CREATE TABLE assignment(\n'
            '    round INTEGER,\n'
            '    pair_or_player INTEGER,\n'
            '    pair_number INTEGER,\n'
            '    table_number INTEGER\n'
            ')'
        ))
        self.query.execute((
            'CREATE TABLE score(\n'
            '    round INTEGER,\n'
            '    table_number INTEGER,\n'
            '    game INTEGER,\n'
            '    pair_or_player INTEGER,\n'
            '    score INTEGER\n'
            ')'
        ))

        self.dbFile = dbFile
        self.setStandardValues()
        self.updateSettings()
        self.__tournamentStarted = None

    def openDb(self, dbFile):
        """Trys to open the existing database dbFile."""

        self.db = sqlite3.connect(dbFile)
        self.query = self.db.cursor()

        # Let's see if we can query the database
        try:
            self.query.execute("""
                SELECT name
                FROM sqlite_master
                WHERE
                    type='table'
                    AND name='config'
                    OR name='players'
                    OR name='assignment'
                    OR name='score'
            """)
        except:
            return (
                'Auf die Datenbank konnte nicht zugegriffen werden. '
                'Die Fehlermeldung war: „{}“'
            ).format(sys.exc_info()[1])

        # Let's see if we have all tables
        tableCount = 0
        for i in self.query:
            tableCount += 1
        if tableCount != 4:
            return (
                'In der Datenbank sind nicht alle nötigen Tabllen vorhanden. '
                'vermutlich ist „{}“ keine Muckturnier-Datenbank!'
            ).format(dbFile)

        # Let's see if we have the correct database version

        self.query.execute('SELECT value FROM config WHERE key = ?', (DB.dbVersion,))

        try:
            dbVersion = int(self.query.fetchone()[0])
        except:
            dbVersion = None

        if dbVersion != MT.dbVersion:
            return (
                'Die Datenbank hat die falsche Version (ist {}, sollte {} sein). '
                'Entweder wurde die Datenbank mit einer älteren Version von muckturnier '
                '(vor 0.5.1) erstellt, oder die Datei ist keine Muckturnier-Datenbank.'
            ).format(dbVersion, MT.dbVersion)

        self.dbFile = dbFile
        self.updateSettings()
        self.__tournamentStarted = None

        return True

    def updateSettings(self):
        """Reads the settings stored in the database to self.settings."""
        try:
            self.query.execute('SELECT * FROM config')
            for entry in self.query:
                self.settings[entry[0]] = entry[1]
        except:
            return

    def setStandardValues(self):
        """Saves a set of standard settings to the database."""
        settings = [
            (DB.muckturnierVersion, MT.muckturnierVersion),
            (DB.dbVersion, MT.dbVersion),
            (DB.tournamentMode, DB.fixedPairs),
            (DB.boogerScore, 21),
            (DB.gamesPerRound, 2)
        ]
        self.query.executemany('INSERT INTO config(key, value) VALUES(?, ?)', settings)
        self.db.commit()

    def saveSettings(self, settings):
        """Saves the current settings to the database."""

        for setting in settings:
            self.query.execute('UPDATE config SET value = ? WHERE key = ?', setting)

        self.db.commit()
        self.updateSettings()

    def reset(self):
        """Erases the database and saves a set of standars settings.
        Same as overwriting an existing database with a new one.
        """
        self.query.execute('DELETE FROM config')
        self.query.execute('DELETE FROM players')
        self.query.execute('DELETE FROM assignment')
        self.query.execute('DELETE FROM score')
        self.setStandardValues()
        self.updateSettings()
        self.__tournamentStarted = False

    def playersString(self, count):
        """Returns a correct number player(s)/pair(s) string."""

        if count == MT.singular:
            if self.settings[DB.tournamentMode] == DB.fixedPairs:
                return 'Paar'
            else:
                return 'Spieler'
        elif count == MT.plural:
            if self.settings[DB.tournamentMode] == DB.fixedPairs:
                return 'Paare'
            else:
                return 'Spieler'

    def players(self):
        """Returns an alphabetically sorted list of all player/pair names."""

        playersList = []
        self.query.execute('SELECT name FROM players ORDER BY name COLLATE NOCASE')

        for player in self.query:
            playersList.append(player[0])

        return playersList

    def necessaryDivisor(self):
        """Returns the number of players/pairs necessary to start the tournament."""
        return 2 + (self.settings[DB.tournamentMode] != DB.fixedPairs) * 2

    def checkName(self, name):
        """Checks if a player/pair name already exists."""

        name = name.lower()

        for player in self.players():
            if player.lower() == name:
                return False

        return True

    def registerPlayers(self, name):
        """Saves the player/pair name to the database."""
        self.query.execute('INSERT INTO players(id, name) VALUES(NULL, ?)', (name,))
        self.db.commit()

    def deletePlayers(self, name):
        """"Deletes the player/pair name from the database."""
        self.query.execute('DELETE FROM players WHERE name = ?', (name,))
        self.db.commit()

    def renamePlayers(self, oldName, newName):
        """Renames the player/pair oldName to newName."""
        self.query.execute('UPDATE players SET name = ? WHERE name = ?', (newName, oldName))
        self.db.commit()

    def playersCount(self):
        """Returns the current number of players/pairs."""
        self.query.execute('SELECT COUNT(id) AS count FROM players')
        return self.query.fetchone()[0]

    def roundsState(self):
        """Returns a dictionary of all rounds determining
        if all scores have already been entered."""

        roundsStateMap = {}

        playersCount = self.playersCount()
        allGames = self.playersCount() * int(self.settings[DB.gamesPerRound])

        roundNumber = 1
        self.query.execute("""
            SELECT round, COUNT(round) AS count FROM score
            GROUP BY round ORDER BY round
        """)

        # Check if each round already contains all games
        for res in self.query:
            roundNumber = res[0]
            if res[1] < allGames:
                roundsStateMap[roundNumber] = False
            else:
                roundsStateMap[roundNumber] = True

        # Be sure to have a state for each round. All data from a round could have been deleted.
        # If we have no data, round 1 will be added as open.
        for i in range(1, roundNumber + 1):
            if not i in roundsStateMap:
                roundsStateMap[i] = False

        return roundsStateMap

    def missingTables(self, round):
        """Returns a list of all tables with a missing score for round round."""

        missingTableNumbers = {}

        # First, we assume we need all tables
        for i in range(1, int(self.playersCount() / self.necessaryDivisor()) + 1):
            missingTableNumbers[i] = True

        # Then, we take away those that already have a score
        self.query.execute("""
            SELECT table_number FROM score WHERE round = ?
            GROUP BY table_number ORDER BY table_number
        """, (str(round),))
        for res in self.query:
            missingTableNumbers.pop(res[0])

        # Finally, we return a list of all keys left (= tables without score)
        return missingTableNumbers.keys()

    def missingPlayers(self, round):
        """Returns a list of all players/pairs with no score in round round."""

        missingPlayersMap = {}

        # Get a list of all players/pairs
        self.query.execute('SELECT name, id FROM players')
        for res in self.query:
            missingPlayersMap[res[1]] = res[0]

        # Get a list of all players/pairs with a score in this round and remove them
        self.query.execute("""
            SELECT pair_or_player FROM score WHERE round = ? GROUP BY pair_or_player
        """, (str(round),))
        for res in self.query:
            missingPlayersMap.pop(res[0])

        # Swap keys and values and return the list
        return dict((value, key) for key, value in missingPlayersMap.items())

    def __insertScore(self, round, table, game, pairOrPlayer, score):
        """Saves a score to the database."""
        self.query.execute("""
            INSERT INTO score(round, table_number, game, pair_or_player, score)
            VALUES(?, ?, ?, ?, ?)
        """, (round, table, game, pairOrPlayer, score))

    def __updateAssignment(self, round, pairOrPlayer, pairNumber, table):
        """Updates the assignment for a pair/player."""
        self.query.execute("""
            INSERT INTO assignment(round, pair_or_player, pair_number, table_number)
            VALUES(?, ?, ?, ?)
        """, (round, pairOrPlayer, pairNumber, table))

    def saveScore(self, round, table, players, score):
        """Saves a score for a pair or both players."""

        for game in sorted(score.keys()):
            for pair in 1, 2:
                self.__insertScore(round, table, game, players[pair][1], score[game][pair])

                if game == 1:
                    # Also update the assignment
                    self.__updateAssignment(round, players[pair][1], pair, table)

                if self.settings[DB.tournamentMode] == DB.singlePlayers:
                    # Also insert the same score for the second player of the pair
                    # (if we're in single player mode)
                    self.__insertScore(round, table, game, players[pair][2], score[game][pair])

                    if game == 1:
                        # Also update the assignment
                        self.__updateAssignment(round, players[pair][2], pair, table)

        self.db.commit()

        if self.__tournamentStarted == False:
            self.__tournamentStarted = True

    def tournamentStarted(self):
        """Checks if the tournament has already started."""
        if self.__tournamentStarted == None:
            self.query.execute('SELECT COUNT(round) FROM SCORE')
            self.__tournamentStarted = self.query.fetchone()[0] != 0
        return self.__tournamentStarted

    def getScore(self, round):
        """Returns all pair and score data for a round."""

        self.query.execute("""
            SELECT
                score.table_number,
                assignment.pair_number,
                players.name,
                score.game,
                score.score
            FROM
                score,
                players,
                assignment
            WHERE
                score.round = ?
                AND score.pair_or_player = players.id
                AND score.pair_or_player = assignment.pair_or_player
                AND score.round = assignment.round
            ORDER BY
                score.table_number,
                assignment.pair_number,
                score.game
        """, (str(round),))

        pairData = {}
        scoreData = {}

        for score in self.query:
            tableNumber = score[0]
            pairNumber = score[1]
            pairName = score[2]
            gameNumber = score[3]
            scoreValue = score[4]

            # Get the pair numbers and names
            if not tableNumber in pairData:
                pairData[tableNumber] = {}
            if not pairNumber in pairData[tableNumber]:
                pairData[tableNumber][pairNumber] = pairName
            else:
                # If we have single players, we have to assemble the pair name
                if not pairName in pairData[tableNumber][pairNumber]:
                    pairData[tableNumber][pairNumber] = '{} / {}'.format(
                        pairData[tableNumber][pairNumber], pairName
                    )

            # Get the scores
            if not tableNumber in scoreData:
                scoreData[tableNumber] = {}
            if not gameNumber in scoreData[tableNumber]:
                scoreData[tableNumber][gameNumber] = {}
            if not pairNumber in scoreData[tableNumber][gameNumber]:
                # In single player mode, this would be done twice
                # as both players have the same score
                scoreData[tableNumber][gameNumber][pairNumber] = scoreValue

        return pairData, scoreData

    def deleteScore(self, round, table):
        """Deletes a score from the database."""

        self.query.execute("""
            DELETE FROM score WHERE round = ? AND table_number = ?
        """, (round, table))
        self.query.execute("""
            DELETE FROM assignment WHERE round = ? AND table_number = ?
        """, (round, table))
        self.db.commit()

        # Perhaps, this was the only score, so re-check if the tournament has started next time
        self.__tournamentStarted = None

    def getAssignment(self, round):
        """Calculates the correct table and pair number for each pair in round round."""

        round -= 1
        allTables = self.playersCount() // self.necessaryDivisor()
        self.query.execute("""
            SELECT pair_or_player, pair_number, table_number FROM assignment WHERE round = ?
        """, (round,))

        assignedTable = {}
        assignedPairNumber = {}
        assignedPair = {}

        for res in self.query:
            if res[1] == 1:
                assignedTable[res[0]] = res[2]
            else:
                assignedTable[res[0]] = res[2] % allTables + 1
            assignedPairNumber[res[0]] = res[1]

        for pairNumber in assignedTable.keys():
            if not assignedTable[pairNumber] in assignedPair:
                assignedPair[assignedTable[pairNumber]] = {}
            assignedPair[assignedTable[pairNumber]][assignedPairNumber[pairNumber]] = pairNumber

        return assignedTable, assignedPairNumber, assignedPair

    def getRanking(self):
        """Calculates the current ranking for all players/pairs."""

        self.query.execute("""
            SELECT
                players.name AS players_name,
                SUM(CASE WHEN score.score = ? THEN 1 ELSE 0 END) AS boogers,
                SUM(CASE WHEN score.score = ? THEN 0 ELSE score.score END) AS goals,
                COUNT(score.score)
            FROM
                players,
                score
            WHERE
                players.id = score.pair_or_player
            GROUP BY
                score.pair_or_player
            ORDER BY
                boogers DESC,
                goals DESC,
                players_name
        """, (self.settings[DB.boogerScore], self.settings[DB.boogerScore]))

        ranking = []

        for rank in self.query:
            ranking.append(rank)

        # Be sure to have data for all players/pairs, in case some of them don't have a score yet
        allPlayers = self.players()
        for players in sorted(allPlayers):
            scoreThere = False
            for singleRanking in ranking:
                if players == singleRanking[0]:
                    scoreThere = True
                    break
            if not scoreThere:
                ranking.append((players, 0, 0, 0))

        return ranking
