#   Copyright (c) 2012-2014 Tobias Leupold <tobias.leupold@web.de>
#
#   This file is part of the muckturnier package.
#
#   This program is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation in version 2 of the License.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
#   for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

import os

from PyQt5.QtCore import Qt, QUrl, QDir
from PyQt5.QtGui import QFont, QDesktopServices, QIcon, QPixmap
from PyQt5.QtWidgets import \
    QMainWindow, QWidget, QVBoxLayout, QLabel, QTabWidget, QMenu, QAction, QApplication, \
    QFileDialog, QMessageBox

from muckturnier.enums import MT, DB
from muckturnier.shared import shared

from muckturnier.SettingsPage import SettingsPage
from muckturnier.Database import Database
from muckturnier.PlayersPage import PlayersPage
from muckturnier.ScorePage import ScorePage
from muckturnier.RankingPage import RankingPage
from muckturnier.AboutWindow import AboutWindow

class MainWindow(QMainWindow):

    """The main window of the program"""

    def __init__(self):
        """Constructs the object."""

        super().__init__()

        self.setup = True

        self.setWindowIcon(QIcon(QPixmap(shared.absolutePath('icon.png'))))

        mainWidget = QWidget(self)
        layout = QVBoxLayout(mainWidget)
        self.setCentralWidget(mainWidget)

        self.headerLabel = QLabel(self)
        self.headerLabel.setAlignment(Qt.AlignHCenter)

        headerFont = self.headerLabel.font()
        headerFont.setPointSize(headerFont.pointSize() * 1.5)
        headerFont.setWeight(QFont.Bold)
        self.headerLabel.setFont(headerFont)
        layout.addWidget(self.headerLabel)

        self.tabWidget = QTabWidget(self)
        layout.addWidget(self.tabWidget)
        self.tabWidget.currentChanged.connect(self.tabChanged)
        self.tabWidget.setEnabled(False)

        # ----------

        self.settingsPage = SettingsPage()
        self.tabWidget.addTab(self.settingsPage, 'Einstellungen')
        self.settingsPage.statusUpdate.connect(self.updateStatus)
        self.settingsPage.settingsSaved.connect(self.settingsSaved)
        self.settingsPage.dbReset.connect(self.dbReset)

        self.playersPage = PlayersPage()
        self.tabWidget.addTab(self.playersPage, 'Paare anmelden')
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.playersPage), False)
        self.playersPage.statusUpdate.connect(self.updateStatus)
        self.playersPage.tournamentCanBeStarted.connect(self.setScoreTabEnabled)

        self.scorePage = ScorePage()
        self.tabWidget.addTab(self.scorePage, 'Ergebnisse eintragen')
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.scorePage), False)
        self.scorePage.tournamentStarted.connect(self.tournamentStarted)
        self.scorePage.statusUpdate.connect(self.updateStatus)

        self.rankingPage = RankingPage()
        self.tabWidget.addTab(self.rankingPage, 'Rangliste')
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.rankingPage), False)
        self.playersPage.playersListChanged.connect(self.rankingPage.reload)
        self.scorePage.scoreChanged.connect(self.rankingPage.reload)
        self.rankingPage.statusUpdate.connect(self.updateStatus)

        # Menu

        fileMenu = self.menuBar().addMenu('Datei')

        newTournamentAction = QAction('Neues Turnier starten', self)
        fileMenu.addAction(newTournamentAction)
        newTournamentAction.triggered.connect(self.startNewTournament)

        loadTournamentAction = QAction('Turnier laden', self)
        fileMenu.addAction(loadTournamentAction)
        loadTournamentAction.triggered.connect(self.loadTournament)

        fileMenu.addSeparator()

        quitAction = QAction('Beenden', self)
        fileMenu.addAction(quitAction)
        quitAction.triggered.connect(self.quit)

        helpMenu = self.menuBar().addMenu('Hilfe')

        handbookAction = QAction('Handbuch zu muckturnier', self)
        helpMenu.addAction(handbookAction)
        handbookAction.triggered.connect(self.showHandbook)

        aboutAction = QAction('Über muckturnier', self)
        helpMenu.addAction(aboutAction)
        aboutAction.triggered.connect(self.showAboutWindow)

        # ----------

        self.setWindowTitle('muckturnier')
        self.updateStatus('Bitte ein neues Turnier starten oder ein bestehendes laden!')

        self.db = Database.instance()
        self.setup = False

    def tabChanged(self, tab):
        """Updates the global header, according to the currently selected tab."""

        if self.setup:
            return

        if self.tabWidget.currentIndex() == self.tabWidget.indexOf(self.playersPage):
            self.playersPage.playersName.setFocus()

        self.headerLabel.setText(self.tabWidget.tabText(tab))


    def startNewTournament(self):
        """Creates a new database after asking for a file name."""

        dbFileName = QFileDialog.getSaveFileName(
            self,
            'Bitte Dateinamen für die Datenbank auswählen',
            'Turnier.db',
            'muckturnier-Datenbaken (*.db)'
        )[0]

        if dbFileName == '':
            return

        # Can we write to it?
        try:
            fp = open(dbFileName, 'a')
        except:
            QMessageBox.critical(self,
                'Neues Turnier starten',
                (
                    'Es bestehen keine Schreibrechte für die Datei „{}“. '
                    'Bitte die Zugriffsrechte überprüfen!'
                ).format(QDir.toNativeSeparators(dbFileName)),
                QMessageBox.Ok
            )
            return
        fp.close()

        self.updateStatus('Lege neue Turnierdatenbank an …')
        QApplication.setOverrideCursor(Qt.WaitCursor)

        self.db.create(dbFileName)
        self.databaseChanged(dbFileName)
        self.dbReset()
        self.settingsPage.resetGui()
        self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.settingsPage))

        self.updateStatus('Turnier {} gestartet'.format(QDir.toNativeSeparators(dbFileName)))
        QApplication.restoreOverrideCursor()

    def loadTournament(self, checked, dbToOpen = None):
        """Loads a database. Either after asking for a file or using the given file name."""

        # The 'checked' value is passed by QAction.triggered()'s connect, so we have to catch it

        if dbToOpen == None:
            dbFileName = QFileDialog.getOpenFileName(self,
                'Bitte eine Turnierdatenbank auswählen',
                '',
                'muckturnier-Datenbaken (*.db)'
            )[0]

            if dbFileName == '':
                return
        else:
            dbFileName = str(dbToOpen)

        # Can we open it?
        try:
            fp = open(dbFileName)
        except:
            QMessageBox.critical(self,
                'Turnier laden',
                (
                    'Laden von Turnierdatei „{}“ ist nicht möglich: '
                    'Die Datei kann nicht geöffnet werden! '
                    'Bitte die Zugriffsrechte überprüfen!'
                ).format(QDir.toNativeSeparators(dbFileName)),
                QMessageBox.Ok
            )
            return
        fp.close()

        # Can we write to it?
        try:
            fp = open(dbFileName, 'a')
        except:
            QMessageBox.critical(self,
                'Turnier laden',
                (
                    'Laden von Turnierdatei „{}“ ist nicht möglich: '
                    'Die Datei ist zwar lesbar, aber schreibgeschützt. '
                    'Es könnten also keine Daten eingegeben werden. '
                    'Bitte die Zugriffsrechte überprüfen!'
                ).format(QDir.toNativeSeparators(dbFileName)),
                QMessageBox.Ok
            )
            return
        fp.close()

        # At least, we can read and write the file. So let's open it
        self.updateStatus('Öffne Turnierdatenbank …')
        QApplication.setOverrideCursor(Qt.WaitCursor)

        openMessage = self.db.openDb(dbFileName)
        if openMessage != True:
            QApplication.restoreOverrideCursor()
            QMessageBox.critical(self,
                'Turnier laden',
                'Laden von Turnierdatei „{}“ ist nicht möglich:<br>{}'.format(
                    QDir.toNativeSeparators(dbFileName), openMessage
                ),
                QMessageBox.Ok
            )
            return

        self.databaseChanged(dbFileName)
        self.settingsPage.setToSaved()

        if self.db.tournamentStarted():
            self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.scorePage))
        else:
            self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.playersPage))

        self.updateStatus('Turnier {} geöffnet'.format(QDir.toNativeSeparators(dbFileName)))
        QApplication.restoreOverrideCursor()

    def quit(self):
        """Terminates the program."""
        QApplication.quit()

    def updateStatus(self, message):
        """Displays a message in the status bar for 5 seconds."""
        self.statusBar().showMessage(message, 5000)

    def databaseChanged(self, dbFileName):
        """Updates the GUI after loading or creating a database."""

        self.setWindowTitle('muckturnier: {}'.format(
            QDir.toNativeSeparators(dbFileName).split(os.sep)[-1]
        ))
        self.tabWidget.setEnabled(True)

        self.settingsPage.reload()

        self.setPlayersTabName()
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.playersPage), True)
        self.playersPage.reload()

        self.checkEnableRankingPage()
        if self.db.tournamentStarted():
            self.rankingPage.reload()

    def dbReset(self):
        """Updates the GUI when resetting the database."""
        self.setPlayersTabName()
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.playersPage), False)
        self.playersPage.reset()
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.scorePage), False)
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.rankingPage), False)

    def settingsSaved(self):
        """Updates the GUI when saving the tournament settings."""
        self.setPlayersTabName()
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.playersPage), True)
        self.playersPage.reload()
        self.tabWidget.setCurrentIndex(self.tabWidget.indexOf(self.playersPage))

    def setPlayersTabName(self):
        """Sets the correct players/pairs tab name."""
        if self.db.tournamentStarted():
            self.tabWidget.setTabText(
                self.tabWidget.indexOf(self.playersPage),
                'Angemeldete {}'.format(self.db.playersString(MT.plural))
            )
        else:
            self.tabWidget.setTabText(
                self.tabWidget.indexOf(self.playersPage),
                '{} anmelden'.format(self.db.playersString(MT.plural))
            )

    def setScoreTabEnabled(self, state):
        """Enables/disables the score tab."""
        self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.scorePage), state)
        if(state == True):
            self.scorePage.reload()

    def tournamentStarted(self):
        """Updates the GUI when the tournament has been started."""
        self.setPlayersTabName()
        self.playersPage.reload()
        self.checkEnableRankingPage()

    def checkEnableRankingPage(self):
        """Enables or disables the ranking page tab."""
        if self.db.tournamentStarted():
            self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.rankingPage), True)
        else:
            self.tabWidget.setTabEnabled(self.tabWidget.indexOf(self.rankingPage), False)

    def showAboutWindow(self):
        """Shows the 'about' window."""
        aboutWindow = AboutWindow(self)
        aboutWindow.show()

    def showHandbook(self):
        """Opens the default browser to display the handbook."""

        readmePath = shared.absolutePath('Readme.htm')
        if not os.path.exists(readmePath):
            QMessageBox.critical(self,
                'Handbuch anzeigen',
                (
                    'Das Handbuch kann nicht angezeigt werden: '
                    'Die Datei „Readme.htm“ wurde nicht gefunden.'
                ),
                QMessageBox.Ok
            )
            return

        QDesktopServices.openUrl(QUrl.fromLocalFile(readmePath))
