#   Copyright (c) 2012-2014 Tobias Leupold <tobias.leupold@web.de>
#
#   This file is part of the muckturnier package.
#
#   This program is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation in version 2 of the License.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
#   for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtWidgets import \
    QWidget, QVBoxLayout, QGroupBox, QHBoxLayout, QLabel, QSizePolicy, QLineEdit, QPushButton, \
    QListWidget, QListWidgetItem, QMessageBox, QTableWidget, QTableWidgetItem, QMenu, QAction, \
    QApplication

from muckturnier.Database import Database

from muckturnier.enums import MT, DB
from muckturnier.shared import shared

class PlayersPage(QWidget):

    """This is the dialog used to register/edit/delete players or pairs."""

    tournamentCanBeStarted = pyqtSignal(object)
    statusUpdate = pyqtSignal(object)
    playersListChanged = pyqtSignal()

    def __init__(self):
        """Constructs the object."""

        super().__init__()

        layout = QVBoxLayout(self)

        # Neue Paare/Spieler anmelden

        self.newPlayersBox = QGroupBox(self)
        self.newPlayersBox.setStyleSheet(MT.groupTitleStyle)
        newPlayersBoxLayout = QHBoxLayout(self.newPlayersBox)
        layout.addWidget(self.newPlayersBox)

        self.nameLabel = QLabel('Name(n):', self)
        self.nameLabel.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        newPlayersBoxLayout.addWidget(self.nameLabel)

        self.playersName = QLineEdit(self)
        newPlayersBoxLayout.addWidget(self.playersName)
        self.playersName.textChanged.connect(self.playersNameChanged)
        self.playersName.returnPressed.connect(self.registerPlayers)

        self.registerButton = QPushButton('anmelden', self)
        self.registerButton.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        self.registerButton.setEnabled(False)
        newPlayersBoxLayout.addWidget(self.registerButton)
        self.registerButton.clicked.connect(self.registerPlayers)

        # Angemeldete Paare/Spieler

        self.playersBox = QGroupBox(self)
        self.playersBox.setStyleSheet(MT.groupTitleStyle)
        playersBoxLayout = QVBoxLayout(self.playersBox)
        layout.addWidget(self.playersBox)

        self.playersList = QTableWidget(0, 1, self)
        self.playersList.horizontalHeader().hide()
        self.playersList.verticalHeader().hide()
        self.playersList.setShowGrid(False)
        self.playersList.setWordWrap(False)
        self.playersList.setSelectionMode(QTableWidget.NoSelection)
        self.playersList.cellDoubleClicked.connect(self.editStart)
        self.playersList.cellChanged.connect(self.editEntered)
        self.playersList.verticalScrollBar().valueChanged.connect(
            self.playersList.resizeColumnsToContents
        )
        self.playersList.setContextMenuPolicy(Qt.CustomContextMenu)
        self.playersList.customContextMenuRequested.connect(self.showDelete)

        self.deleteMenu = QMenu(self)
        self.deleteAction = QAction(self)
        self.deleteMenu.addAction(self.deleteAction)
        self.deleteAction.triggered.connect(self.deletePlayers)

        playersBoxLayout.addWidget(self.playersList)

        self.stateLabel = QLabel(self)
        layout.addWidget(self.stateLabel)

        self.db = Database.instance()

    def reload(self):
        """Refreshes all data to display"""

        self.newPlayersBox.show()
        self.newPlayersBox.setTitle('Neue {} anmelden'.format(self.db.playersString(MT.plural)))
        self.playersBox.setTitle('Angemeldete {}'.format(self.db.playersString(MT.plural)))

        if self.db.settings[DB.tournamentMode] == DB.fixedPairs:
            self.nameLabel.setText('Name(n):')
        else:
            self.nameLabel.setText('Name:')

        self.playersName.setFocus()
        self.playersList.clear()

        playersCount = 0
        allPlayers = self.db.players()
        self.playersList.clear()
        self.playersList.setRowCount(len(allPlayers))
        color = 1

        self.playersList.blockSignals(True)

        for player in allPlayers:
            color += 1
            color = color % 2
            self.playersList.setItem(playersCount, 0, shared.cell(player, MT.editable, color))
            playersCount += 1

        self.playersList.resizeColumnsToContents()

        self.playersList.blockSignals(False)

        if playersCount == 0:
            state = 'Keine {} angemeldet'.format(self.db.playersString(MT.plural))
        elif playersCount == 1:
            state = '1 {} angemeldet'.format(self.db.playersString(MT.singular))
        else:
            state = '{} {} angemeldet'.format(playersCount, self.db.playersString(MT.plural))

        necessaryDivisor = self.db.necessaryDivisor()
        if playersCount % necessaryDivisor != 0:
            state += (
                '. Die Anzahl der {} muss durch {} teilbar sein, '
                'um mit dem Turnier beginnen zu können!'
            ).format(self.db.playersString(MT.plural), necessaryDivisor)
            self.tournamentCanBeStarted.emit(False)
        else:
            if playersCount > 0:
                self.tournamentCanBeStarted.emit(True)

        self.stateLabel.setText(state)

        if self.db.tournamentStarted():
            self.newPlayersBox.hide()

    def reset(self):
        """Resets the dialog."""
        self.playersList.clear()
        self.newPlayersBox.show()

    def playersNameChanged(self, name):
        """Checks if a player/pair name is empty and enables/disables the 'register' button"""
        self.registerButton.setEnabled(name.strip() != '')

    def registerPlayers(self):
        """Checks a new player/pair name and saves it if everything is okay."""

        if not self.registerButton.isEnabled():
            return

        name = shared.simplify(self.playersName.text())

        if not self.db.checkName(name):
            QMessageBox.critical(self,
                self.newPlayersBox.title(),
                'Der Name „{}“ ist bereits vergeben!'.format(name),
                QMessageBox.Ok
            )
            return

        self.statusUpdate.emit('Melde {} an …'.format(self.db.playersString(MT.singular)))
        QApplication.setOverrideCursor(Qt.WaitCursor)

        self.db.registerPlayers(name)
        self.playersName.setText('')
        self.playersName.setFocus()
        self.reload()

        QApplication.restoreOverrideCursor()
        self.statusUpdate.emit('{} „{}“ angemeldet'.format(
            self.db.playersString(MT.singular), name
        ))

    def editStart(self):
        """Caches the original player/pair name when editing it."""
        self.originalPlayersName = self.playersList.currentItem().text()

    def __setPlayersName(self, name):
        """Updates the displayed player/pair's name."""
        self.playersList.blockSignals(True)
        self.playersList.currentItem().setText(name)
        self.playersList.blockSignals(False)

    def editEntered(self):
        """Checks a changed player/pair name and saves it if everything is okay."""

        name = shared.simplify(self.playersList.currentItem().text())

        if name == '':
            self.__setPlayersName(self.originalPlayersName)
            return

        if not self.db.checkName(name):
            QMessageBox.critical(self,
                '{} umbenennen nicht möglich'.format(self.db.playersString(MT.singular)),
                'Der Name „{}“ ist bereits vergeben!'.format(name),
                QMessageBox.Ok
            )
            self.__setPlayersName(self.originalPlayersName)
            return

        self.statusUpdate.emit('Benenne {} um …'.format(self.db.playersString(MT.singular)))
        QApplication.setOverrideCursor(Qt.WaitCursor)

        self.db.renamePlayers(self.originalPlayersName, name)
        self.__setPlayersName(name)

        QApplication.restoreOverrideCursor()
        self.statusUpdate.emit(
            '{} „{}“ umbenannt zu „{}“'.format(
                self.db.playersString(MT.singular), self.originalPlayersName, name
            )
        )

        self.playersListChanged.emit()
        self.reload()

    def showDelete(self, point):
        """Shows a 'delete' popup menu."""

        if self.db.tournamentStarted():
            return

        selectedPlayers = self.playersList.currentItem()
        if selectedPlayers == None:
            return

        self.deleteAction.setText('„{}“ löschen'.format(selectedPlayers.text()))
        self.deleteMenu.exec_(self.playersList.mapToGlobal(point))

    def deletePlayers(self):
        """Deletes a player/pair after asking."""

        toDelete = self.playersList.currentItem().text()

        reply = QMessageBox.warning(
            self,
            '{} löschen'.format(self.db.playersString(MT.singular)),
            'Soll {} „{}“ wirklich gelöscht werden?'.format(
                self.db.playersString(MT.singular), toDelete
            ),
            QMessageBox.Yes | QMessageBox.No
        )

        if reply != QMessageBox.Yes:
            return;

        self.db.deletePlayers(toDelete)
        self.reload()
