#   Copyright (c) 2012-2014 Tobias Leupold <tobias.leupold@web.de>
#
#   This file is part of the muckturnier package.
#
#   This program is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation in version 2 of the License.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
#   for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from html import escape
from time import strftime

from PyQt5.QtCore import Qt, pyqtSignal, QUrl, QDir
from PyQt5.QtGui import QDesktopServices
from PyQt5.QtWidgets import \
    QWidget, QVBoxLayout, QGroupBox, QLabel, QTableWidget, QTableWidgetItem, QHBoxLayout, \
    QLineEdit, QPushButton, QFileDialog, QApplication, QCheckBox, QMessageBox

from muckturnier.Database import Database

from muckturnier.enums import DB, MT
from muckturnier.shared import shared

class RankingPage(QWidget):

    """This shows the current ranking of all players/pairs."""

    statusUpdate = pyqtSignal(object)

    def __init__(self):
        """Constructs the object."""

        super().__init__()

        layout = QVBoxLayout(self)

        self.rankingBox = QGroupBox(self)
        self.rankingBox.setStyleSheet(MT.groupTitleStyle)
        rankingBoxLayout = QVBoxLayout(self.rankingBox)
        layout.addWidget(self.rankingBox)

        self.interimLabel = QLabel()
        rankingBoxLayout.addWidget(self.interimLabel)
        self.interimLabel.hide()

        self.ranking = QTableWidget(self)
        self.ranking.verticalHeader().hide()
        self.ranking.setShowGrid(False)
        self.ranking.setWordWrap(False)
        self.ranking.setSelectionMode(QTableWidget.NoSelection)
        self.ranking.setFocusPolicy(Qt.NoFocus)
        self.ranking.setColumnCount(4)
        rankingBoxLayout.addWidget(self.ranking)

        self.journalBox = QGroupBox('Protokoll', self)
        self.journalBox.setStyleSheet(MT.groupTitleStyle)
        journalBoxLayout = QHBoxLayout(self.journalBox)
        layout.addWidget(self.journalBox)

        journalBoxLayout.addWidget(QLabel('Titel:', self))

        self.titleEdit = QLineEdit('Muckturnier', self)
        journalBoxLayout.addWidget(self.titleEdit)
        self.titleEdit.textChanged.connect(self.checkJournalTitle)

        self.saveJournalButton = QPushButton('Protokoll erstellen', self)
        journalBoxLayout.addWidget(self.saveJournalButton)
        self.saveJournalButton.clicked.connect(self.saveJournal)

        self.openJournalCheck = QCheckBox('und anzeigen', self)
        self.openJournalCheck.setChecked(Qt.Checked)
        journalBoxLayout.addWidget(self.openJournalCheck)

        self.journalBox.hide()

        self.db = Database.instance()

    def reload(self):
        """Refreshes all data and displays it."""

        roundsState = self.db.roundsState()
        incompleteRounds = []
        for round in sorted(roundsState.keys()):
            if not roundsState[round]:
                incompleteRounds.append(round)

        missingRounds = len(incompleteRounds)

        if missingRounds != 0:
            lastRound = incompleteRounds[missingRounds - 1]
            self.journalBox.hide()
        else:
            lastRound = len(roundsState)
            self.journalBox.show()

        if missingRounds > 0:
            self.rankingBox.setTitle('Rangliste im Verlauf der {}. Runde'.format(lastRound))

            interimText = (
                '<p><b>Achtung!</b> Das ist nur ein Zwischenergebnis. '
                'Es fehlen noch Ergebnisse aus '
            )
            if missingRounds == 1:
                interimText += 'Runde {}.'.format(incompleteRounds[0])
            elif missingRounds == 2:
                interimText += 'den Runden {} und {}.'.format(
                    incompleteRounds[0], incompleteRounds[1]
                )
            else:
                interimText += 'den Runden '
                for missing in range(missingRounds - 1):
                    interimText += '{}, '.format(incompleteRounds[missing])
                interimText = interimText[:-2] + ' und {}.'.format(lastRound)
            interimText += '</p>'
            interimText += (
                '<p>Paare, von denen noch Spielstände fehlen, sind '
                '<span style="color:red">rot</span> markiert.</p>'
            )

            self.interimLabel.setText(interimText)
            self.interimLabel.show()

        else:
            self.allRounds = len(roundsState)
            if self.allRounds == 1:
                self.rankingBox.setTitle('Rangliste nach einer Runde')
            else:
                if self.allRounds == 2:
                    roundsStr = 'zwei'
                elif self.allRounds == 3:
                    roundsStr = 'drei'
                elif self.allRounds == 4:
                    roundsStr = 'vier'
                elif self.allRounds == 5:
                    roundsStr = 'fünf'
                elif self.allRounds == 6:
                    roundsStr = 'sechs'
                elif self.allRounds == 7:
                    roundsStr = 'sieben'
                elif self.allRounds == 8:
                    roundsStr = 'acht'
                elif self.allRounds == 9:
                    roundsStr = 'neun'
                elif self.allRounds == 10:
                    roundsStr = 'zehn'
                elif self.allRounds == 11:
                    roundsStr = 'elf'
                elif self.allRounds == 12:
                    roundsStr = 'zwölf'
                else:
                    roundsStr = str(self.allRounds)
                self.rankingBox.setTitle('Rangliste nach {} Runden'.format(roundsStr))

            self.interimLabel.hide()

        ranking = self.db.getRanking()
        allGames = lastRound * int(self.db.settings[DB.gamesPerRound])

        self.ranking.clearContents()
        self.ranking.setRowCount(0)
        self.ranking.setHorizontalHeaderLabels(
            ('Platz', self.db.playersString(MT.singular), 'Bobbl', 'Tore')
        )

        self.ranking.setRowCount(len(ranking))
        rank = 0
        trueRank = 0
        prevRank = 0
        prevBoogers = 0
        prevGoals = 0

        color = 1
        for score in ranking:
            trueRank += 1
            if score[1] != prevBoogers or score[2] != prevGoals:
                rank = trueRank
            else:
                rank = prevRank

            if rank != prevRank:
                color += 1
                color = color % 2

            prevRank = rank
            prevBoogers = score[1]
            prevGoals = score[2]

            if score[3] != allGames:
                textColor = Qt.red
            else:
                textColor = None

            self.ranking.setItem(
                trueRank - 1, 0, shared.cell(rank, MT.notEditable, color, textColor)
            )
            self.ranking.setItem(
                trueRank - 1, 1, shared.cell(score[0], MT.notEditable, color, textColor)
            )
            self.ranking.setItem(
                trueRank - 1, 2, shared.cell(score[1], MT.notEditable, color, textColor)
            )
            self.ranking.setItem(
                trueRank - 1, 3, shared.cell(score[2], MT.notEditable, color, textColor)
            )

        self.ranking.resizeColumnsToContents()

    def checkJournalTitle(self):
        """Enables or disables the 'create journal' button if a title is given."""
        if shared.simplify(self.titleEdit.text()) == '':
            self.saveJournalButton.setEnabled(False)
        else:
            self.saveJournalButton.setEnabled(True)

    def saveJournal(self):
        """Saves a HTML journal to a file."""

        title = escape(shared.simplify(self.titleEdit.text()))

        journalFileName = QFileDialog.getSaveFileName(
            self,
            'Bitte Dateinamen für das Protokoll auswählen',
            title + '.htm',
            'HTML-Dateien (*.htm)'
        )[0]

        if journalFileName == '':
            return

        # Can we write to it?
        try:
            fp = open(journalFileName, 'w', encoding = 'utf-8')
        except:
            QMessageBox.critical(self,
                'Protokoll speichern',
                (
                    'Es bestehen keine Schreibrechte für die Datei „{}“. '
                    'Bitte die Zugriffsrechte überprüfen oder einen '
                    'anderen Dateinamen auswählen!'
                ).format(QDir.toNativeSeparators(journalFileName)),
                QMessageBox.Ok
            )
            return

        self.statusUpdate.emit('Erstelle Protokoll …')
        QApplication.setOverrideCursor(Qt.WaitCursor)

        # Create a header
        fp.write((
            '<?xml version="1.0" encoding="UTF-8"?>\n\n'
            '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"\n'
            '   "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">\n\n'
            '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">\n\n'
            '<head>\n\n'
            '<title>{}</title>\n\n'
            '<meta http-equiv="content-type" content="text/html; charset=UTF-8" />\n\n'
            '<style type="text/css">\n'
            'table {{ border-spacing: 0px; }}\n'
            'td {{ padding: 0.2em 1em 0.2em; }}\n'
            '.line {{ border-top: 1px solid black; }}\n'
            '</style>\n\n'
            '</head>\n\n'
            '<body>\n\n'
            '<h1>{}</h1>\n\n'
        ).format(title, title))

        # Add tournament meta info

        fp.write((
            '<h2>Allgemeine Daten</h2>\n\n'
            '<h3>muckturnier</h3>\n\n'
            '<table>\n'
            '<tr><td>Protokoll erstellt am:</td><td>{} um {} Uhr</td></tr>\n'
        ).format(strftime('%d.%m.%Y'), strftime('%H.%M')))

        if MT.muckturnierVersion == self.db.settings[DB.muckturnierVersion]:
            fp.write((
                '<tr>'
                '<td>Turnierdatenbank und Protokoll erstellt mit:</td>'
                '<td>muckturnier {}</td>'
                '</tr>\n'
            ).format(MT.muckturnierVersion))
        else:
            fp.write((
                '<tr><td>Protokoll erstellt mit:</td><td>muckturnier {}</td></tr>\n'
                '<tr><td>Turnierdatenbank erstellt mit:</td><td>muckturnier {}</td></tr>\n'
            ).format(
                MT.muckturnierVersion,
                self.db.settings[DB.muckturnierVersion]
            ))

        fp.write((
            '</table>\n\n'
            '<h3>Turnier</h3>\n\n'
            '<table>\n'
            '<tr><td>Teilnehmende {}:</td><td>{}</td></tr>\n'
            '<tr><td>Gespielte Runden:</td><td>{}</td></tr>\n'
            '</table>\n\n'
        ).format(
            self.db.playersString(MT.plural), self.ranking.rowCount(),
            self.allRounds
        ))

        # Add the ranking

        fp.write((
            '<h2>{}</h2>\n\n'
            '<table>\n'
            '<thead>\n'
                '<tr>'
                    '<td><b>Platz</b></td>'
                    '<td><b>{}</b></td>'
                    '<td><b>Bobbl</b>'
                    '</td><td><b>Tore</b></td>'
                '</tr>\n'
            '</thead>\n'
            '<tbody>\n'
        ).format(self.rankingBox.title(), self.db.playersString(MT.singular)))

        prevRank = -1
        style = ''

        for row in range(self.ranking.rowCount()):
            rank = self.ranking.item(row, 0).text()

            if rank != prevRank:
                style = ' class="line"'
                prevRank = rank
            else:
                style = ''

            fp.write((
                '<tr>'
                '<td{}>{}</td>'
            ).format(style, rank))

            for i in range(1, 4):
                fp.write('<td{}>{}</td>'.format(style, escape(self.ranking.item(row, i).text())))

            fp.write('</tr>\n')

        fp.write((
            '</tbody>\n'
            '</table>\n\n'
        ))

        # Add all round data

        for round in range(1, self.allRounds + 1):
            fp.write((
                '<h2>Spielergebnisse der {}. Runde</h2>\n\n'
                '<table>\n'
                '<thead>\n'
                '<tr>'
                    '<td><b>Tisch</b></td>'
                    '<td><b>Paar</b></td>'
            ).format(round))

            for game in range(1, int(self.db.settings[DB.gamesPerRound]) + 1):
                fp.write('<td><b>{}. Bobbl</b></td>'.format(game))

            fp.write((
                '</tr>\n'
                '</thead>\n'
                '<tbody>\n'
            ))

            pairData, scoreData = self.db.getScore(round)

            for table in sorted(scoreData.keys()):

                row = {
                    1: [],
                    2: []
                }

                row[1].append(table)

                for i in 1, 2:
                    row[i].append(pairData[table][i])

                for game in sorted(scoreData[table].keys()):
                    for i in 1, 2:
                        row[i].append(
                            shared.boogify(scoreData[table][game][i], self.db.settings[DB.boogerScore])
                        )

                fp.write('<tr><td rowspan="2" class="line">{}</td>'.format(row[1].pop(0)))
                for dat in row[1]:
                    fp.write('<td class="line">{}</td>'.format(escape(dat)))
                fp.write('</tr>\n<tr>')
                for dat in row[2]:
                    fp.write('<td>{}</td>'.format(escape(dat)))
                fp.write('</tr>\n')

            fp.write((
                '</tbody>\n'
                '</table>\n\n'
            ))

        # Close HTML
        fp.write((
            '</body>\n\n'
            '</html>\n'
        ))

        fp.close()

        if self.openJournalCheck.isChecked():
            QDesktopServices.openUrl(QUrl.fromLocalFile(journalFileName))

        QApplication.restoreOverrideCursor()
        self.statusUpdate.emit(
            'Protokoll in „{}“ gespeichert'.format(QDir.toNativeSeparators(journalFileName))
        )
