#   Copyright (c) 2012-2014 Tobias Leupold <tobias.leupold@web.de>
#
#   This file is part of the muckturnier package.
#
#   This program is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation in version 2 of the License.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
#   for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtWidgets import \
    QWidget, QVBoxLayout, QGroupBox, QHBoxLayout, QLabel, QSizePolicy, QComboBox, QGridLayout, \
    QCheckBox, QFrame, QScrollArea, QRadioButton, QSpinBox, QPushButton, QMessageBox, \
    QTableWidget, QTableWidgetItem, QHeaderView, QMenu, QAction, QApplication

from muckturnier.Database import Database

from muckturnier.enums import MT, DB
from muckturnier.shared import shared

class ScorePage(QWidget):

    """This is the interface for saving a score."""

    tournamentStarted = pyqtSignal()
    statusUpdate = pyqtSignal(object)
    scoreChanged = pyqtSignal()

    def __init__(self):
        """Constructs the object"""

        super().__init__()

        self.disableChange = True

        layout = QVBoxLayout(self)

        # Runde

        roundBox = QGroupBox('Runde', self)
        roundBox.setStyleSheet(MT.groupTitleStyle)
        roundBoxLayout = QHBoxLayout(roundBox)
        layout.addWidget(roundBox)

        roundLabel = QLabel('Runde:', self)
        roundLabel.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        roundBoxLayout.addWidget(roundLabel)

        self.roundSelect = QComboBox(self)
        self.roundSelect.setSizeAdjustPolicy(QComboBox.AdjustToContents)
        self.roundSelect.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        roundBoxLayout.addWidget(self.roundSelect)
        self.roundSelect.currentIndexChanged.connect(self.openRound)

        roundBoxLayout.addStretch()

        # Tisch und Paare/Spieler

        self.playersBox = QGroupBox('Tisch und Paare', self)
        self.playersBox.setStyleSheet(MT.groupTitleStyle)
        playersBoxLayout = QGridLayout(self.playersBox)
        layout.addWidget(self.playersBox)

        tableLabel = QLabel('Tischnummer:', self)
        tableLabel.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        playersBoxLayout.addWidget(tableLabel, 0, 0, 2, 1)

        self.tableSelect = QComboBox(self)
        self.tableSelect.setSizeAdjustPolicy(QComboBox.AdjustToContents)
        self.tableSelect.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        playersBoxLayout.addWidget(self.tableSelect, 0, 1, 2, 1)
        self.tableSelect.currentIndexChanged.connect(self.tableChanged)

        self.playersSelect = {}
        for i in 1, 2:
            self.playersSelect[i] = {}

        for players in 1, 2:
            playersLabel = QLabel('Paar {}'.format(players), self)
            playersLabel.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
            playersBoxLayout.addWidget(playersLabel, players - 1, 2);

            for player in 1, 2:
                self.playersSelect[players][player] = QComboBox(self)
                self.playersSelect[players][player].setSizeAdjustPolicy(
                    QComboBox.AdjustToContents
                )
                self.playersSelect[players][player].setSizePolicy(
                    QSizePolicy.Maximum, QSizePolicy.Maximum
                )
                playersBoxLayout.addWidget(
                    self.playersSelect[players][player], players - 1, 2 + player
                )

        self.playersSelect[1][1].currentIndexChanged.connect(self.pair1Changed)
        self.playersSelect[2][1].currentIndexChanged.connect(self.pair2Changed)

        self.playersSelect[1][2].currentIndexChanged.connect(self.pair34Changed)
        self.playersSelect[2][2].currentIndexChanged.connect(self.pair34Changed)

        self.autoSelect = QCheckBox('Tischnummer bzw. Paare\nautomatisch auswählen', self)
        self.autoSelect.setChecked(Qt.Checked)
        playersBoxLayout.addWidget(self.autoSelect, 0, 5, 2, 1)

        # This works as a spacer so that the place is filled if self.autoSelect is not shown
        playersBoxLayout.addWidget(QLabel(self), 0, 6)

        # Spielergebnis eintragen

        self.scoreBox = QGroupBox('Spielergebnis', self)
        self.scoreBox.setStyleSheet(MT.groupTitleStyle)
        self.scoreBoxLayout = QVBoxLayout(self.scoreBox)
        self.scoreBox.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Maximum)
        layout.addWidget(self.scoreBox)

        self.scoreBoxScroll = QScrollArea(self.scoreBox)
        self.scoreBoxScroll.setFrameShape(QFrame.NoFrame)
        self.scoreBoxScroll.setStyleSheet('QScrollArea { background-color: transparent; }')
        self.scoreBoxLayout.addWidget(self.scoreBoxScroll)

        self.scoreBoxWidget = QWidget(self)

        self.saveScoreButton = QPushButton('Ergebnis\neintragen', self)
        self.saveScoreButton.clicked.connect(self.saveScore)

        # (Bisherige) Ergebnisse der Runde

        self.roundSummaryBox = QGroupBox(self)
        self.roundSummaryBox.setStyleSheet(MT.groupTitleStyle)
        roundSummaryBoxLayout = QVBoxLayout(self.roundSummaryBox)
        layout.addWidget(self.roundSummaryBox)

        self.scoreList = QTableWidget(self)
        self.scoreList.verticalHeader().hide()
        self.scoreList.setShowGrid(False)
        self.scoreList.setWordWrap(False)
        self.scoreList.setSelectionMode(QTableWidget.NoSelection)
        self.scoreList.setFocusPolicy(Qt.NoFocus)
        self.scoreList.setContextMenuPolicy(Qt.CustomContextMenu)
        self.scoreList.customContextMenuRequested.connect(self.showDelete)

        roundSummaryBoxLayout.addWidget(self.scoreList)

        self.deleteMenu = QMenu(self)
        self.deleteAction = QAction(self)
        self.deleteMenu.addAction(self.deleteAction)
        self.deleteAction.triggered.connect(self.deleteScore)

        # ----------

        self.db = Database.instance()

    def reload(self, lastRound = None):
        """Refreshes all data and displays it."""

        self.disableChange = True

        self.saveScoreButton.setEnabled(False)

        # Show the needed input fields for pairs or players
        if self.db.settings[DB.tournamentMode] == DB.fixedPairs:
            self.playersSelect[1][2].hide()
            self.playersSelect[2][2].hide()
            self.autoSelect.show()
        else:
            self.playersSelect[1][2].show()
            self.playersSelect[2][2].show()
            self.autoSelect.setChecked(False)
            self.autoSelect.hide()

        # Display the correct number of game widgets

        self.scoreBoxWidget.deleteLater()
        self.scoreBoxWidget = QWidget(self)
        self.scoreBoxWidgetLayout = QHBoxLayout(self.scoreBoxWidget)

        for i in range(1, int(self.db.settings[DB.gamesPerRound]) + 1):
            newGame = GameWidget(i, int(self.db.settings[DB.boogerScore]))
            self.scoreBoxWidgetLayout.addWidget(newGame)

        self.scoreBoxWidgetLayout.addWidget(self.saveScoreButton)
        self.saveScoreButton.setEnabled(False)
        self.scoreBoxScroll.setWidget(self.scoreBoxWidget)
        self.scoreBoxWidget.setAutoFillBackground(False)

        # Fill the round select

        allRounds = 0
        firstOpenRound = -1

        self.roundSelect.clear()
        self.roundsState = self.db.roundsState()
        for round in self.roundsState:
            if self.roundsState[round] == True:
                self.roundSelect.addItem('{} (abgeschlossen)'.format(round), round)
            else:
                self.roundSelect.addItem('{} (offen)'.format(round), round)
                if firstOpenRound == -1:
                    firstOpenRound = self.roundSelect.count() - 1
            allRounds += 1

        if firstOpenRound == -1:
            firstOpenRound = 0

        if lastRound != None:
            firstOpenRound = lastRound - 1

        # Add one for a new round
        allRounds += 1
        self.roundSelect.addItem('{} (neue Runde)'.format(allRounds), allRounds)
        self.roundsState[allRounds] = False

        self.roundSelect.setCurrentIndex(firstOpenRound)
        self.openRound(firstOpenRound)
        self.disableChange = False

        # Update the score widget
        self.scoreList.setColumnCount(int(self.db.settings[DB.gamesPerRound]) + 2)

        headerLabels = []
        headerLabels.append('Tisch')
        headerLabels.append('Paare')
        for i in range(int(self.db.settings[DB.gamesPerRound])):
            headerLabels.append('{}. Bobbl'.format(i + 1))
        headerLabels.append('')

        self.scoreList.setHorizontalHeaderLabels(headerLabels)
        self.updateScore()

    def openRound(self, round):
        """Selects a round to work on."""

        if round == -1:
            return

        self.disableChange = True

        round = self.roundSelect.itemData(round)

        if self.roundsState[round] == True:
            self.playersBox.hide()
            self.scoreBox.hide()
        else:
            self.playersBox.show()
            self.scoreBox.show()

        # Insert all missing tables
        self.tableSelect.clear()
        for table in self.db.missingTables(round):
            self.tableSelect.addItem(str(table), table)

        # Insert all missing players/pairs

        for i in 1, 2:
            self.playersSelect[i][1].clear()
            if self.db.settings[DB.tournamentMode] == DB.singlePlayers:
                self.playersSelect[i][2].clear()

        missingPlayers = self.db.missingPlayers(round)
        for missingPlayer in sorted(missingPlayers.keys()):
            for i in 1, 2:
                self.playersSelect[i][1].addItem(missingPlayer, missingPlayers[missingPlayer])
                if self.db.settings[DB.tournamentMode] == DB.singlePlayers:
                    self.playersSelect[i][2].addItem(missingPlayer, missingPlayers[missingPlayer])

        if self.roundsState[round] == True:
            self.roundSummaryBox.setTitle('Alle Ergebnisse aus Runde {}'.format(round))
        else:
            self.roundSummaryBox.setTitle(
                'Bisher eingetragene Ergebnisse in Runde {}'.format(round)
            )
        self.updateScore()

        self.assignedTable, self.assignedPairNumber, self.assignedPair = self.db.getAssignment(round)

        self.disableChange = False

        self.tableChanged(0)

    def updateScore(self):
        """Updates the display of the current score data."""

        pairData, scoreData = self.db.getScore(
            self.roundSelect.itemData(self.roundSelect.currentIndex())
        )

        self.scoreList.setRowCount(len(scoreData) * 2)
        row = -1

        color = 1

        for table in sorted(scoreData.keys()):
            color += 1
            color = color % 2

            row += 1
            self.scoreList.setItem(row, 0, shared.cell(table, MT.notEditable, color))
            self.scoreList.setSpan(row, 0, 2, 1)

            self.scoreList.setItem(row, 1, shared.cell(pairData[table][1], MT.notEditable, color))
            row += 1
            self.scoreList.setItem(row, 1, shared.cell(pairData[table][2], MT.notEditable, color))

            for game in sorted(scoreData[table].keys()):
                self.scoreList.setItem(
                    row - 1, game + 1, shared.cell(
                        shared.boogify(scoreData[table][game][1],
                        self.db.settings[DB.boogerScore]
                    ), MT.notEditable, color)
                )
                self.scoreList.setItem(
                    row, game + 1, shared.cell(
                    shared.boogify(
                        scoreData[table][game][2],
                        self.db.settings[DB.boogerScore]
                    ), MT.notEditable, color)
                )

        self.scoreList.resizeColumnsToContents()

    def tableChanged(self, table):
        """Triggered when a table is selected. Sets the respective pairs if requested."""

        if self.disableChange or not self.autoSelect.isChecked():
            return

        table = self.tableSelect.itemData(table)
        if table in self.assignedPair:
            if 1 in self.assignedPair[table]:
                target = self.playersSelect[1][1].findData(self.assignedPair[table][1])
                if target != -1:
                    self.playersSelect[1][1].setCurrentIndex(target)
            if 2 in self.assignedPair[table]:
                target = self.playersSelect[2][1].findData(self.assignedPair[table][2])
                if target != -1:
                    self.playersSelect[2][1].setCurrentIndex(target)

    def pair1Changed(self, pair):
        """Helper function if a pair/player has been selected."""
        self.pairChanged(1, pair)

    def pair2Changed(self, pair):
        """Helper function if a pair/player has been selected."""
        self.pairChanged(2, pair)

    def pair34Changed(self, pair):
        """Helper function if a player has been selected."""
        self.pairChanged(None, pair)

    def pairChanged(self, pairNumber, pair):
        """Check the pair/player selection and select the respective opponent pair if requested."""

        if self.disableChange:
            return

        if self.autoSelect.isChecked():
            pairId = self.playersSelect[pairNumber][1].itemData(pair)
            if pairId in self.assignedPairNumber:
                if pairNumber != self.assignedPairNumber[pairId]:
                    QMessageBox.warning(
                        self,
                        'Die Paarauswahl passt nicht zur Auslosung',
                        (
                            '<p>Als Paar 1 wurde gerade das Paar „{}“ ausgewählt.</p>'
                            '<p>'
                            'Laut der Auslosung (bzw. dem Ergebnis der letzten Runde) '
                            'is dieses Paar aber ein Paar {}!'
                            '</p>'
                        ).format(
                            self.playersSelect[pairNumber][1].currentText(),
                            self.assignedPairNumber[pairId]
                        )
                    )
                else:
                    if pairId in self.assignedTable:
                        self.tableSelect.setCurrentIndex(
                            self.tableSelect.findData(self.assignedTable[pairId])
                        )

        savePossible = False

        if self.db.settings[DB.tournamentMode] == DB.fixedPairs:
            if self.playersSelect[1][1].currentIndex() != self.playersSelect[2][1].currentIndex():
                savePossible = True
        else:
            selectedPlayers = []
            for i in 1, 2:
                for j in 1, 2:
                    if not self.playersSelect[i][j].currentIndex() in selectedPlayers:
                        selectedPlayers.append(self.playersSelect[i][j].currentIndex())
            if len(selectedPlayers) == 4:
                savePossible = True

        self.saveScoreButton.setEnabled(savePossible)

    def saveScore(self):
        """Saves the currently entered score."""

        self.statusUpdate.emit('Speichere Spielergebnis …')
        QApplication.setOverrideCursor(Qt.WaitCursor)

        updatePlayersPage = not self.db.tournamentStarted()

        score = {}
        game = 0
        for gameWidget in self.scoreBoxWidget.findChildren(GameWidget):
            game += 1
            score[game] = {}
            for pair in 1, 2:
                score[game][pair] = gameWidget.score[pair].value()

        players = {}
        for i in 1, 2:
            players[i] = {}
            for j in 1, 2:
                players[i][j] = self.playersSelect[i][j].itemData(
                    self.playersSelect[i][j].currentIndex()
                )

        round = self.roundSelect.itemData(self.roundSelect.currentIndex())
        table = self.tableSelect.itemData(self.tableSelect.currentIndex())
        self.db.saveScore(round, table, players, score)

        if updatePlayersPage:
            self.tournamentStarted.emit()

        self.scoreChanged.emit()
        self.reload(round)

        QApplication.restoreOverrideCursor()
        self.statusUpdate.emit(
            'Spielergebnis für Tisch {} in Runde {} gespeichert'.format(table, round)
        )

    def showDelete(self, point):
        """Displays a pop-up menu to delete a score."""

        selectedScore = self.scoreList.currentItem()
        if selectedScore == None:
            return

        row = selectedScore.row()
        if row % 2 != 0:
            row = row - 1
        table = self.scoreList.item(row, 0).text()

        point.setY(point.y() + self.scoreList.horizontalHeader().height())
        self.deleteAction.setText('Ergebnis für Tisch {} löschen'.format(table))
        self.deleteAction.setData(table)
        self.deleteMenu.exec_(self.scoreList.mapToGlobal(point))

    def deleteScore(self):
        """Delete a score after asking."""

        table = self.deleteAction.data()

        reply = QMessageBox.warning(
            self,
            'Spielergebnis löschen',
            'Soll das Spielergebnis von Tisch {} wirklich gelöscht werden?'.format(table),
            QMessageBox.Yes | QMessageBox.No
        )

        if reply != QMessageBox.Yes:
            return;

        self.statusUpdate.emit('Lösche Spielergebnis …')
        QApplication.setOverrideCursor(Qt.WaitCursor)

        round = self.roundSelect.itemData(self.roundSelect.currentIndex())
        self.db.deleteScore(round, table)
        self.reload()

        if not self.db.tournamentStarted():
            # Re-emitting this will cause the players page to be updated again,
            # as it checks for db.tournamentStarted() itself
            self.tournamentStarted.emit()

        self.scoreChanged.emit()

        QApplication.restoreOverrideCursor()
        self.statusUpdate.emit(
            'Spielergebnis für Tisch {} in Runde {} gelöscht'.format(round, table)
        )

class GameWidget(QWidget):

    """This is the input mask for the score of a booger."""

    def __init__(self, game, boogerScore):
        """Constructs the object."""

        super(GameWidget, self).__init__()

        self.boogerScore = boogerScore
        self.setAutoFillBackground(False)

        layout = QGridLayout(self)

        layout.addWidget(QLabel('<b>{}. Bobbl</b>'.format(game), self), 0, 0)

        self.score = {}
        self.booger = {}
        for i in 1, 2:
            layout.addWidget(QLabel('Paar {}:'.format(i), self), i, 0)

            self.score[i] = QSpinBox(self)
            if i == 1:
                self.score[i].setMaximum(self.boogerScore - 1)
            else:
                self.score[i].setMaximum(self.boogerScore)
            layout.addWidget(self.score[i], i, 1)

            self.booger[i] = QRadioButton(self)
            layout.addWidget(self.booger[i], i, 2)
            self.booger[i].clicked.connect(self.toggleScore)

        self.booger[1].setChecked(Qt.Checked)
        self.score[2].setValue(self.boogerScore)
        self.score[2].setEnabled(False)

    def toggleScore(self):
        """Enables/disables the correct spinboxes, according to the booger selection."""
        for i in 1, 2:
            if self.booger[i].isChecked():
                self.score[i].setMaximum(self.boogerScore - 1)
                self.score[i].setValue(0)
                self.score[i].setEnabled(True)
                self.score[i].setFocus()
                self.score[i].selectAll()
            else:
                self.score[i].setMaximum(self.boogerScore)
                self.score[i].setValue(self.boogerScore)
                self.score[i].setEnabled(False)
