#   Copyright (c) 2012-2014 Tobias Leupold <tobias.leupold@web.de>
#
#   This file is part of the muckturnier package.
#
#   This program is free software; you can redistribute it and/or modify it
#   under the terms of the GNU General Public License as published by the Free
#   Software Foundation in version 2 of the License.
#
#   This program is distributed in the hope that it will be useful, but
#   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
#   or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
#   for more details.
#
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

from PyQt5.QtCore import Qt, pyqtSignal, QDir
from PyQt5.QtWidgets import \
    QWidget, QVBoxLayout, QGroupBox, QGridLayout, QRadioButton, QSizePolicy, QLabel, QSpinBox, \
    QPushButton, QHBoxLayout, QMessageBox, QApplication

from muckturnier.Database import Database

from muckturnier.enums import MT, DB
from muckturnier.shared import shared

class SettingsPage(QWidget):

    """The settings dialog."""

    statusUpdate = pyqtSignal(object)
    settingsSaved = pyqtSignal()
    dbReset = pyqtSignal()

    tournamentModeDescription = {}
    tournamentModeDescription[DB.fixedPairs] = (
        '<p>Die Spieler in einem Paar ändern sich im Verlauf des Turniers nicht.</p>'
        '<p>'
        'Wenn vor Beginn für jedes Paar ein Tisch und die Rolle „Paar 1“ und „Paar 2“ ausgelost '
        'werden, Paar 1 immer sitzen bleibt und Paar 2 nach jeder Runde einen Tisch weiterrückt, '
        'können die passenden Paare und Tische nach der ersten Runde automatisch gesetzt werden.'
        '</p>'
        '<p>'
        'Altervativ kann auch pro Runde ausgelost werden, welche Paare gegeneinander spielen.'
        '</p>'
    )
    tournamentModeDescription[DB.singlePlayers] = (
        '<p>'
        'Es werden einzelne Spieler angemeldet. '
        'In jeder Runde werden die Paare und Tische neu ausgelost.'
        '</p>'
    )

    def __init__(self):
        """Constructs the object."""

        super().__init__()

        layout = QVBoxLayout(self)

        # Datenbank

        self.databaseBox = QGroupBox('Datenbank', self)
        self.databaseBox.setStyleSheet(MT.groupTitleStyle)
        databaseBoxLayout = QGridLayout(self.databaseBox)
        layout.addWidget(self.databaseBox)

        dbFileHeaderLabel = QLabel('Datenbankdatei:', self)
        dbFileHeaderLabel.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        databaseBoxLayout.addWidget(dbFileHeaderLabel, 0, 0)
        self.dbFileLabel = QLabel(self)
        self.dbFileLabel.setTextInteractionFlags(Qt.TextSelectableByMouse)
        databaseBoxLayout.addWidget(self.dbFileLabel, 0, 1)

        mtVersionHeaderLabel = QLabel('Erstellt mit:', self)
        mtVersionHeaderLabel.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        databaseBoxLayout.addWidget(mtVersionHeaderLabel, 1, 0)
        self.mtVersionLabel = QLabel(self)
        self.mtVersionLabel.setTextInteractionFlags(Qt.TextSelectableByMouse)
        databaseBoxLayout.addWidget(self.mtVersionLabel, 1, 1)

        self.databaseBox.hide()

        # Turniertyp

        tournamentTypeBox = QGroupBox('Turniertyp', self)
        tournamentTypeBox.setStyleSheet(MT.groupTitleStyle)
        tournamentTypeLayout = QVBoxLayout(tournamentTypeBox)
        layout.addWidget(tournamentTypeBox)

        self.pairs = QRadioButton('Feste Paare', self)
        self.pairs.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        tournamentTypeLayout.addWidget(self.pairs)
        self.pairs.clicked.connect(self.updateTournamentModeDescription)

        self.singlePlayers = QRadioButton('Einzelne Spieler', self)
        self.singlePlayers.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        tournamentTypeLayout.addWidget(self.singlePlayers)
        self.singlePlayers.clicked.connect(self.updateTournamentModeDescription)

        self.tournamentModeHeaderLabel = QLabel(self)
        self.tournamentModeHeaderLabel.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        self.tournamentModeHeaderLabel.setStyleSheet('QLabel { text-decoration: underline; }')
        tournamentTypeLayout.addWidget(self.tournamentModeHeaderLabel)

        # Adds additional space
        tournamentTypeLayout.addWidget(QLabel(self))

        self.tournamentModeLabel = QLabel(self)
        self.tournamentModeLabel.setText(self.tournamentModeDescription[DB.fixedPairs])
        self.tournamentModeLabel.setWordWrap(True)
        self.tournamentModeLabel.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        tournamentTypeLayout.addWidget(self.tournamentModeLabel)

        # Spieleinstellungen

        gameSettingsBox = QGroupBox('Spieleinstellungen', self)
        gameSettingsBox.setStyleSheet(MT.groupTitleStyle)
        gameSettingsMainLayout = QHBoxLayout(gameSettingsBox)
        gameSettingsLayout = QGridLayout()
        gameSettingsMainLayout.addLayout(gameSettingsLayout)
        layout.addWidget(gameSettingsBox)

        gameSettingsLayout.addWidget(QLabel('Punkte pro Bobbl:', self), 0, 0)

        self.boogerScore = QSpinBox(self)
        self.boogerScore.setMinimum(1)
        gameSettingsLayout.addWidget(self.boogerScore, 0, 1)

        self.boogerScoreLabel = QLabel(self)
        gameSettingsLayout.addWidget(self.boogerScoreLabel, 0, 2)

        gameSettingsLayout.addWidget(QLabel('Bobbl pro Runde:', self), 1, 0)

        self.gamesPerRound = QSpinBox(self)
        self.gamesPerRound.setMinimum(1)
        gameSettingsLayout.addWidget(self.gamesPerRound, 1, 1)

        self.gamesPerRoundLabel = QLabel(self)
        gameSettingsLayout.addWidget(self.gamesPerRoundLabel, 1, 2)

        gameSettingsMainLayout.addStretch()

        # Turniereinstellungen speichern

        saveSettingsWidget = QWidget(self)
        layout.addWidget(saveSettingsWidget)
        saveSettingsLayout = QHBoxLayout(saveSettingsWidget)

        self.resetDatabase = QPushButton('Datenbank zurücksetzen', self)
        self.resetDatabase.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        self.resetDatabase.setVisible(False)
        saveSettingsLayout.addWidget(self.resetDatabase)
        self.resetDatabase.clicked.connect(self.slotResetDatabase)

        self.saveSettings = QPushButton('Turniereinstellungen speichern', self)
        self.saveSettings.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)
        saveSettingsLayout.addWidget(self.saveSettings)
        self.saveSettings.clicked.connect(self.slotSaveSettings)

        saveSettingsLayout.addStretch()

        layout.addStretch()

        self.db = Database.instance()
        self.resetGui()

    def updateTournamentModeDescription(self):
        """Display the correct description for the tournament mode."""
        if self.pairs.isChecked():
            self.tournamentModeLabel.setText(self.tournamentModeDescription[DB.fixedPairs])
            self.tournamentModeHeaderLabel.setText('Feste Paare:')
        else:
            self.tournamentModeLabel.setText(self.tournamentModeDescription[DB.singlePlayers])
            self.tournamentModeHeaderLabel.setText('Einzelne Spieler:')

    def slotResetDatabase(self):
        """Resets the database after asking."""

        reply = QMessageBox.warning(
            self,
            'Datenbank zurücksetzen',
            'Sollen wirklich alle Einstellungen und Ergebnisse gelöscht werden?',
            QMessageBox.Yes | QMessageBox.No
        )

        if reply != QMessageBox.Yes:
            return;

        self.statusUpdate.emit('Setzte Turnierdatenbank zurück …')
        QApplication.setOverrideCursor(Qt.WaitCursor)

        self.db.reset()
        self.resetGui()
        self.reload()
        self.dbReset.emit()

        QApplication.restoreOverrideCursor()
        self.statusUpdate.emit('Turnier zurückgesetzt')

    def slotSaveSettings(self):
        """Saves the selected settings."""

        self.statusUpdate.emit('Speichere Turniereinstellungen …')
        QApplication.setOverrideCursor(Qt.WaitCursor)

        settings = []

        if self.pairs.isChecked():
            settings.append((DB.fixedPairs, DB.tournamentMode))
        else:
            settings.append((DB.singlePlayers, DB.tournamentMode))

        settings.append((str(self.boogerScore.value()), DB.boogerScore))
        settings.append((str(self.gamesPerRound.value()), DB.gamesPerRound))

        self.db.saveSettings(settings)
        self.setToSaved()
        self.settingsSaved.emit()

        QApplication.restoreOverrideCursor()
        self.statusUpdate.emit('Turniereinstellungen gespeichert')

    def resetGui(self):
        """Resets the GUI."""

        self.resetDatabase.hide()
        self.saveSettings.show()

        self.tournamentModeHeaderLabel.setText('Feste Paare:')
        self.tournamentModeHeaderLabel.hide()
        self.tournamentModeLabel.setText(self.tournamentModeDescription[DB.fixedPairs])
        self.pairs.show()
        self.singlePlayers.show()

        self.boogerScoreLabel.hide()
        self.boogerScore.show()
        self.gamesPerRoundLabel.hide()
        self.gamesPerRound.show()

        self.databaseBox.hide()

    def reload(self):
        """Sets all displayed values accoding to their db values."""

        if self.db.settings[DB.tournamentMode] == DB.fixedPairs:
            self.pairs.setChecked(Qt.Checked)
            self.singlePlayers.setChecked(Qt.Unchecked)
        else:
            self.pairs.setChecked(Qt.Unchecked)
            self.singlePlayers.setChecked(Qt.Checked)

        self.updateTournamentModeDescription()
        self.boogerScore.setValue(int(self.db.settings[DB.boogerScore]))
        self.gamesPerRound.setValue(int(self.db.settings[DB.gamesPerRound]))

    def setToSaved(self):
        """Change the GUI from 'select' to 'display'."""

        self.resetDatabase.show()
        self.saveSettings.hide()

        self.boogerScoreLabel.setText(str(self.boogerScore.value()))
        self.boogerScoreLabel.show()
        self.boogerScore.hide()
        self.gamesPerRoundLabel.setText(str(self.gamesPerRound.value()))
        self.gamesPerRoundLabel.show()
        self.gamesPerRound.hide()

        self.tournamentModeHeaderLabel.show()
        self.pairs.hide()
        self.singlePlayers.hide()

        self.databaseBox.show()
        self.dbFileLabel.setText(QDir.toNativeSeparators(self.db.dbFile))
        self.mtVersionLabel.setText('muckturnier ' + self.db.settings[DB.muckturnierVersion])
